@@ -5,14 +5,22 @@
 
 #include "compression.hh"
 #include "check.hh"
+#include "endian.hh"
+#include "debug.hh"
 
 namespace Compression {
 
-EnDecoder::EnDecoder() { }
-EnDecoder::~EnDecoder() { }
+EnDecoder::EnDecoder()
+{
+}
 
-CompressionMethod::~CompressionMethod() { }
+EnDecoder::~EnDecoder()
+{
+}
 
+CompressionMethod::~CompressionMethod()
+{
+}
 
 // LZMA
 
@@ -72,8 +80,17 @@ class LZMAEncoder : public LZMAEnDecoder
 public:
   LZMAEncoder()
   {
-    uint32_t preset = 6; // TODO: make this customizable, although 6 seems to be
-                         // the best option
+    uint32_t preset = 6;
+    lzma_ret ret = lzma_easy_encoder( &strm, preset, LZMA_CHECK_CRC64 );
+    CHECK( ret == LZMA_OK, "lzma_easy_encoder error: %d", (int) ret );
+  }
+
+  LZMAEncoder( Config const & config )
+  {
+    uint32_t compressionLevel = config.GET_STORABLE( lzma, compression_level );
+    uint32_t preset = ( compressionLevel > 9 ) ?
+      ( compressionLevel - 10 ) | LZMA_PRESET_EXTREME :
+      compressionLevel;
     lzma_ret ret = lzma_easy_encoder( &strm, preset, LZMA_CHECK_CRC64 );
     CHECK( ret == LZMA_OK, "lzma_easy_encoder error: %d", (int) ret );
   }
@@ -92,6 +109,11 @@ public:
 class LZMACompression : public CompressionMethod
 {
 public:
+  sptr<EnDecoder> createEncoder( Config const & config ) const
+  {
+    return new LZMAEncoder( config );
+  }
+
   sptr<EnDecoder> createEncoder() const
   {
     return new LZMAEncoder();
@@ -105,7 +127,6 @@ public:
   std::string getName() const { return "lzma"; }
 };
 
-
 // LZO
 
 // liblzo implements a lot of algorithms "for unlimited backward compatibility"
@@ -232,7 +253,7 @@ public:
     {
       // data has been encoded or decoded, remaining output is in accDataOut
       // -> copy to output
-      if (availOut > 0 && accDataOut.size() - posInAccDataOut > 0)
+      if ( availOut > 0 && accDataOut.size() - posInAccDataOut > 0 )
       {
         size_t sz = availOut;
         if ( sz > accDataOut.size() - posInAccDataOut )
@@ -273,7 +294,7 @@ private:
     // we use our own buffer
     size_t bufferSize = suggestOutputSize( dataIn, availIn );
     do {
-      accDataOut.resize(bufferSize);
+      accDataOut.resize( bufferSize );
 
       size_t outputSize;
       //TODO doc says we mustn't modify the pointer returned by data()...
@@ -292,7 +313,13 @@ private:
   }
 };
 
+#ifdef __APPLE__
+#include <machine/endian.h>
+#elif __FreeBSD__
+#include <sys/endian.h>
+#else
 #include <endian.h>
+#endif
 
 // like NoStreamEnDecoder, but also adds the uncompressed size before the stream
 //NOTE You should make sure that the compression function doesn't overwrite any
@@ -308,7 +335,6 @@ protected:
   virtual bool doProcessNoSize( const char* dataIn, size_t availIn,
       char* dataOut, size_t availOut, size_t& outputSize ) =0;
 
-
   bool shouldTryWith( const char* dataIn, size_t availIn, size_t availOut )
   {
     return suggestOutputSize( dataIn, availIn ) <= availOut;
@@ -384,7 +410,6 @@ protected:
   virtual bool doProcessNoSize( const char* dataIn, size_t availIn,
       char* dataOut, size_t availOut, size_t& outputSize ) =0;
 
-
   bool shouldTryWith( const char*, size_t, size_t availOut )
   {
     // If the compression doesn't use any spaces...
@@ -439,7 +464,6 @@ protected:
   }
 };
 
-
 #ifdef HAVE_LIBLZO
 
 #include <lzo/lzo1x.h>
@@ -468,9 +492,9 @@ class LZO1X_1_Compression;
 class LZO1X_1_Encoder : public NoStreamAndUnknownSizeEncoder
 {
   const LZO1X_1_Compression* compression;
-  static size_t calcMaxCompressedSize(size_t availIn);
+  static size_t calcMaxCompressedSize( size_t availIn );
 public:
-  LZO1X_1_Encoder(const LZO1X_1_Compression* compression)
+  LZO1X_1_Encoder( const LZO1X_1_Compression* compression )
   {
     this->compression = compression;
   }
@@ -495,13 +519,19 @@ class LZO1X_1_Compression : public CompressionMethod
     }
   }
 public:
-  sptr<EnDecoder> createEncoder() const
+  sptr< EnDecoder > createEncoder( Config const & config ) const
   {
     init();
     return new LZO1X_1_Encoder(this);
   }
 
-  sptr<EnDecoder> createDecoder() const
+  sptr< EnDecoder > createEncoder() const
+  {
+    init();
+    return new LZO1X_1_Encoder(this);
+  }
+
+  sptr< EnDecoder > createDecoder() const
   {
     init();
     return new LZO1X_1_Decoder();
@@ -509,7 +539,6 @@ public:
 
   std::string getName() const { return "lzo1x_1"; }
 
-
   lzo_voidp getWorkmem( size_t size ) const
   {
     return new char[size];
@@ -527,7 +556,6 @@ public:
 
 bool LZO1X_1_Compression::initialized = false;
 
-
 size_t LZO1X_1_Encoder::calcMaxCompressedSize( size_t availIn )
 {
   // It seems that lzo1x_1_compress does NOT check whether the buffer is big enough.
@@ -580,23 +608,157 @@ bool LZO1X_1_Encoder::doProcessNoSize( const char* dataIn, size_t availIn,
 
 #endif  // HAVE_LIBLZO
 
+// Zero compression
+
+class ZeroEnDecoder : public EnDecoder
+{
+protected:
+  size_t size, offset, toCopy, left, BackUp;
+
+public:
+  ZeroEnDecoder()
+  {
+    BackUp = 0;
+  }
+
+  size_t getAvailableInput()
+  {
+    return BackUp;
+  }
+
+  size_t getAvailableOutput()
+  {
+    return BackUp;
+  }
+};
+
+class ZeroEncoder : public ZeroEnDecoder
+{
+  const void * payload;
+  void * data;
+
+  void setInput( const void * data, size_t size )
+  {
+    payload = data;
+    offset = toCopy = 0;
+    left = size;
+  }
+
+  void setOutput( void * data, size_t size )
+  {
+    this->data = data;
+    this->size = size;
+  }
+
+  bool process( bool finish )
+  {
+    toCopy = ( left > size ) ? size : left;
+
+    memcpy( data, ( char * ) payload + offset, toCopy );
+    //dPrintf( "data:\n|%s|\n", payload + offset );
+
+    offset += toCopy;
+    left -= toCopy;
+
+    if ( 0 >= left )
+    {
+      if ( toCopy < size )
+        BackUp = size - toCopy;
+      return true;
+    }
+
+    return false;
+  }
+
+public:
+  ZeroEncoder()
+  {
+  }
+
+  ZeroEncoder( Config const & config )
+  {
+  }
+};
+
+class ZeroDecoder : public ZeroEnDecoder
+{
+  void * payload;
+  const void * data;
+
+  void setInput( const void * data, size_t size )
+  {
+    this->data = data;
+    this->size = size;
+  }
+
+  void setOutput( void * data, size_t size )
+  {
+    payload = data;
+    offset = toCopy = 0;
+    left = size;
+  }
+
+  bool process( bool finish )
+  {
+    toCopy = ( left > size ) ? size : left;
+
+    memcpy( ( char * ) payload + offset, data, toCopy );
+    //dPrintf( "data:\n|%s|\n", data );
+
+    offset += toCopy;
+    left -= toCopy;
+
+    if ( 0 >= left )
+    {
+      if ( toCopy < size )
+        BackUp = size - toCopy;
+      return true;
+    }
+
+    return false;
+  }
+};
+
+class ZeroCompression : public CompressionMethod
+{
+public:
+  sptr<EnDecoder> createEncoder( Config const & config ) const
+  {
+    return new ZeroEncoder( config );
+  }
+
+  sptr<EnDecoder> createEncoder() const
+  {
+    return new ZeroEncoder();
+  }
+
+  sptr<EnDecoder> createDecoder() const
+  {
+    return new ZeroDecoder();
+  }
+
+  std::string getName() const { return "zero"; }
+};
 
 // register them
 
-static const_sptr<CompressionMethod> const compressions[] = {
+const_sptr< CompressionMethod > const CompressionMethod::compressions[] = {
   new LZMACompression(),
 # ifdef HAVE_LIBLZO
   new LZO1X_1_Compression(),
 # endif
+  new ZeroCompression(),
   // NULL entry marks end of list. Don't remove it!
   NULL
 };
 
-const_sptr<CompressionMethod> CompressionMethod::defaultCompression = compressions[0];
+const_sptr< CompressionMethod > CompressionMethod::selectedCompression =
+  compressions[ 0 ];
 
-const_sptr<CompressionMethod> CompressionMethod::findCompression( const std::string& name, bool optional )
+const_sptr< CompressionMethod > CompressionMethod::findCompression(
+    const std::string& name, bool optional )
 {
-  for ( const const_sptr<CompressionMethod>* c = compressions+0; *c; ++c )
+  for ( const const_sptr<CompressionMethod>* c = compressions + 0; *c; ++c )
   {
     if ( (*c)->getName() == name )
     {
@@ -611,9 +773,15 @@ const_sptr<CompressionMethod> CompressionMethod::findCompression( const std::str
 }
 
 // iterator over compressions
+CompressionMethod::iterator::iterator( const const_sptr< CompressionMethod > * ptr ):
+  ptr( ptr )
+{
+}
 
-CompressionMethod::iterator::iterator( const const_sptr<CompressionMethod>* ptr ) : ptr(   ptr) { }
-CompressionMethod::iterator::iterator( const iterator&                 it ) : ptr(it.ptr) { }
+CompressionMethod::iterator::iterator( const iterator & it ):
+  ptr( it.ptr )
+{
+}
 
 CompressionMethod::iterator& CompressionMethod::iterator::operator =( const iterator& it )
 {
