@@ -2,3 +2,10 @@
 // Part of ZBackup. Licensed under GNU GPLv2 or later + OpenSSL, see LICENSE
 
 bool verboseMode = true;
+
+#ifndef NDEBUG
+#ifdef HAVE_LIBUNWIND
+#include "debug.hh"
+
+#endif
+#endif
