@@ -3,6 +3,7 @@
 
 #include "tmp_mgr.hh"
 
+#include <sys/stat.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include "dir.hh"
@@ -44,6 +45,8 @@ sptr< TemporaryFile > TmpMgr::makeTemporaryFile()
   string name( Dir::addPath( path, "XXXXXX") );
 
   int fd = mkstemp( &name[ 0 ] );
+  if ( fchmod ( fd, S_IRUSR | S_IWUSR | S_IRGRP ) != 0 )
+    throw exCantCreate( path );
 
   if ( fd == -1 || close( fd ) != 0 )
     throw exCantCreate( path );
